"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ServiceCollection = void 0;
var Errors_1 = require("../Errors");
var Lifetime_1 = require("../Lifetime");
var ServiceProvider_1 = require("../ServiceProvider");
var mockUtils_1 = require("./mockUtils");
var ServiceCollection = /** @class */ (function () {
    function ServiceCollection() {
        Object.defineProperty(this, "lifetimes", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: {}
        });
    }
    Object.defineProperty(ServiceCollection, "extractSelector", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function (options) {
            switch (typeof options) {
                case 'function':
                    var proxy = new Proxy({}, { get: function (t, p) { return p.toString(); } });
                    return options(proxy);
                case 'symbol':
                    return options.toString();
                case 'string':
                    return options;
                default:
                    throw new Error("Name selector could not match anything");
            }
        }
    });
    Object.defineProperty(ServiceCollection.prototype, "replaceSingleton", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function (options, selector) {
            this.replace(Lifetime_1.Singleton, options, selector);
        }
    });
    Object.defineProperty(ServiceCollection.prototype, "replaceTransient", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function (options, selector) {
            this.replace(Lifetime_1.Transient, options, selector);
        }
    });
    Object.defineProperty(ServiceCollection.prototype, "replaceScoped", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function (options, selector) {
            this.replace(Lifetime_1.Scoped, options, selector);
        }
    });
    Object.defineProperty(ServiceCollection.prototype, "replace", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function (Lifetime, dependency, selector) {
            this.remove(selector);
            this.add(Lifetime, dependency, selector);
        }
    });
    Object.defineProperty(ServiceCollection.prototype, "tryAddSingleton", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function (options, selector) {
            this.tryAdd(Lifetime_1.Singleton, options, selector);
        }
    });
    Object.defineProperty(ServiceCollection.prototype, "tryAddTransient", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function (options, selector) {
            this.tryAdd(Lifetime_1.Transient, options, selector);
        }
    });
    Object.defineProperty(ServiceCollection.prototype, "tryAddScoped", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function (options, selector) {
            this.tryAdd(Lifetime_1.Scoped, options, selector);
        }
    });
    Object.defineProperty(ServiceCollection.prototype, "tryAdd", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function (Lifetime, dependency, selector) {
            try {
                this.add(Lifetime, dependency, selector);
            }
            catch (e) {
                var type = e.type;
                if (type !== Errors_1.DependencyErrorType.Duplicate)
                    throw e;
            }
        }
    });
    Object.defineProperty(ServiceCollection.prototype, "addStateful", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function (Constructor, selector) {
            var key = ServiceCollection.extractSelector(selector).toString();
            var last = { name: "".concat(key, "@constructor") };
            function next(scope) {
                scope[key] = scope[key] || 1;
                return scope[key]++;
            }
            return this.addTransient({
                factory: function (_, original) {
                    var _a;
                    var stateful = new ServiceProvider_1.ScopedContext(original);
                    var _b = (_a = original.lastSingleton) !== null && _a !== void 0 ? _a : {}, isSingleton = _b.isSingleton, name = _b.name;
                    var singleton = name ? "".concat(String(name), "@") : '';
                    var track = {
                        name: "".concat(singleton).concat(key, "@creator#").concat(next(original.scope)),
                        isSingleton: isSingleton,
                    };
                    return {
                        create: function (props) {
                            var inParent = original.depth;
                            var context = inParent ? original : stateful;
                            if (inParent)
                                context.enter(last);
                            context.enter(track);
                            var result = new Constructor(context.proxy, props);
                            context.leave(track);
                            if (inParent)
                                context.leave(last);
                            return result;
                        },
                    };
                },
            }, selector);
        }
    });
    Object.defineProperty(ServiceCollection.prototype, "addSingleton", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function (options, selector) {
            this.add(Lifetime_1.Singleton, options, selector);
        }
    });
    Object.defineProperty(ServiceCollection.prototype, "addTransient", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function (options, selector) {
            this.add(Lifetime_1.Transient, options, selector);
        }
    });
    Object.defineProperty(ServiceCollection.prototype, "addScoped", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function (options, selector) {
            this.add(Lifetime_1.Scoped, options, selector);
        }
    });
    Object.defineProperty(ServiceCollection.prototype, "add", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function (Lifetime, dependency, selector) {
            var name = ServiceCollection.extractSelector(selector);
            if (name in this.lifetimes)
                throw new Errors_1.DuplicateDependencyError(name);
            var factory = this.extractDependency(dependency);
            this.lifetimes[name] = new Lifetime(name, factory);
        }
    });
    Object.defineProperty(ServiceCollection.prototype, "get", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function (selector) {
            var key = ServiceCollection.extractSelector(selector);
            return this.lifetimes[key];
        }
    });
    Object.defineProperty(ServiceCollection.prototype, "remove", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function (selector) {
            var result = this.get(selector);
            if (result)
                delete this.lifetimes[result.name];
            return result;
        }
    });
    Object.defineProperty(ServiceCollection.prototype, "build", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function () {
            var lifetimes = this.cloneLifetimes();
            return new ServiceProvider_1.ServiceProvider(lifetimes);
        }
    });
    Object.defineProperty(ServiceCollection.prototype, "buildMock", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function (mock) {
            if (mock === void 0) { mock = {}; }
            var provider = this.build();
            (0, mockUtils_1.proxyLifetimes)(provider.lifetimes, mock || {});
            return provider;
        }
    });
    Object.defineProperty(ServiceCollection.prototype, "extractDependency", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function (Option) {
            if (typeof Option === 'function')
                return function (p) { return new Option(p); };
            return Option.factory;
        }
    });
    Object.defineProperty(ServiceCollection.prototype, "cloneLifetimes", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function () {
            return Object.entries(this.lifetimes)
                .map(function (_a) {
                var key = _a[0], value = _a[1];
                return ({
                    name: key,
                    lifetime: value.clone(),
                });
            })
                .reduce(function (a, _a) {
                var name = _a.name, lifetime = _a.lifetime;
                a[name] = lifetime;
                return a;
            }, {});
        }
    });
    return ServiceCollection;
}());
exports.ServiceCollection = ServiceCollection;
