"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.proxyValue = exports.proxyLifetime = exports.proxyLifetimes = void 0;
var Errors_1 = require("../Errors");
function proxyLifetimes(lifetimes, mock) {
    Object.values(lifetimes)
        .forEach(function (lifetime) {
        var setup = mock[lifetime.name];
        // @ts-ignore
        lifetimes[lifetime.name] = proxyLifetime(lifetime, setup);
    });
}
exports.proxyLifetimes = proxyLifetimes;
function proxyLifetime(lifetime, mock) {
    return new Proxy(lifetime, {
        get: function (target, prop) {
            var name = lifetime.name.toString();
            if (prop !== 'provide')
                return target[prop];
            return function (context) {
                if (context.depth === 1)
                    return target.provide(context);
                switch (typeof mock) {
                    case 'undefined':
                        return proxyValue(name, {});
                    case 'function':
                        return proxyValue(name, mock(context.proxy, context));
                    default:
                        return proxyValue(name, mock);
                }
            };
        },
    });
}
exports.proxyLifetime = proxyLifetime;
function proxyValue(name, obj) {
    // noinspection JSUnusedGlobalSymbols
    return new Proxy(obj, {
        get: function (target, prop) {
            var _a;
            if ((prop in target) || ((_a = Object.getOwnPropertyDescriptor(target, prop)) === null || _a === void 0 ? void 0 : _a.get))
                return target[prop];
            throw new Errors_1.ShouldBeMockedDependencyError(name, prop.toString(), 'get');
        },
        set: function (target, prop, value) {
            var _a;
            if ((prop in target) || ((_a = Object.getOwnPropertyDescriptor(target, prop)) === null || _a === void 0 ? void 0 : _a.set)) {
                target[prop] = value;
                return true;
            }
            throw new Errors_1.ShouldBeMockedDependencyError(name, prop.toString(), 'set');
        },
    });
}
exports.proxyValue = proxyValue;
