"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ScopedContext = void 0;
var Errors_1 = require("../Errors");
var ServiceCollection_1 = require("../ServiceCollection");
var ScopedContext = /** @class */ (function () {
    function ScopedContext(parent) {
        Object.defineProperty(this, "lifetimes", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "proxy", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "scope", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "singletons", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: []
        });
        Object.defineProperty(this, "lookup", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: {}
        });
        Object.defineProperty(this, "ordered", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: []
        });
        var self = this;
        this.lifetimes = parent.lifetimes;
        this.scope = (parent instanceof ScopedContext) ? parent.scope : {};
        this.proxy = new Proxy({}, {
            get: function (target, prop) { return self.provide(prop); },
        });
    }
    Object.defineProperty(ScopedContext.prototype, "depth", {
        get: function () {
            return this.ordered.length;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(ScopedContext.prototype, "lastSingleton", {
        get: function () {
            return this.singletons[this.singletons.length - 1];
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(ScopedContext.prototype, "enter", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function (lifetime) {
            if (lifetime.name in this.lookup)
                throw new Errors_1.CircularDependencyError(lifetime.name, this.ordered.map(function (e) { return e.name; }));
            this.ordered.push(lifetime);
            if (lifetime.isSingleton)
                this.singletons.push(lifetime);
            this.lookup[lifetime.name] = lifetime;
        }
    });
    Object.defineProperty(ScopedContext.prototype, "leave", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function (lifetime) {
            var last = this.ordered.pop();
            if (last === null || last === void 0 ? void 0 : last.isSingleton)
                this.singletons.pop();
            delete this.lookup[lifetime.name];
        }
    });
    Object.defineProperty(ScopedContext.prototype, "provide", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function (selector) {
            var lifetime = this.getLifetime(selector);
            this.enter(lifetime);
            var result = lifetime.provide(this);
            this.leave(lifetime);
            return result;
        }
    });
    Object.defineProperty(ScopedContext.prototype, "getLifetime", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function (selector) {
            var key = ServiceCollection_1.ServiceCollection.extractSelector(selector);
            var lifetime = this.lifetimes[key];
            if (!lifetime)
                throw new Errors_1.ExistenceDependencyError(key);
            return lifetime;
        }
    });
    return ScopedContext;
}());
exports.ScopedContext = ScopedContext;
